"""The Pyrex (that is, the C) part of fastcharmap.
"""

__author__  = "Tony Nelson"
__email__   = "tonynelson@georgeanelson.com"
__url__     = "http://georgeanelson.com/fastcharmap.htm"
__copyright__ = "Copyright 2005 by George A. Nelson.  All rights reserved."
__license__ = "LGPL"
__date__    = "16 Oct 2005"
__version__ = "0.1.1"

# This file is part of fastcharmap.
# 
# Fastcharmap is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
# 
# Fastcharmap is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with fastcharmap; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

import codecs

# Rather than "from fastcharmap import ...", fastcharmap shoves its own (mutable)
# objects into these, so we'll be connected to the right fastcharmap even if it
# has a different name, such as __main__, or from importing via a different path.
_DictHook = None
_make_decode_cache = _make_encode_cache = None
_orig_decode = _orig_encode = None
_chatty = None

cdef extern from "Python.h":
    ctypedef int Py_UNICODE
    object PyUnicode_FromUnicode(Py_UNICODE *u, int len)
    object PyUnicode_FromObject(object)
    Py_UNICODE * PyUnicode_AsUnicode(object) except NULL
    int PyUnicode_GetSize(object) except -1
    object PyString_FromStringAndSize(char *s, int len)
    int PyArg_ParseTuple(object args, char *format, ...) except 0


def _decode(input, errors, decoding_map):
    cdef unsigned char *s
    cdef Py_UNICODE *u, *m, uc
    cdef int i, n, j
    
    try:
        cache = decoding_map._cache
    except:
        if _chatty[0] >= 2:
            print "_orig_decode (not hooked)"
        return _orig_decode(input, errors, decoding_map)
    
    if cache is None:
        cache = _make_decode_cache(decoding_map)
        decoding_map._cache = cache
    
    try:
        m = PyUnicode_AsUnicode(cache)
        if PyUnicode_GetSize(cache) != 256:
            raise Exception
    except:
        if _chatty[0] >= 2:
            print "_orig_decode (no cache)"
        return _orig_decode(input, errors, decoding_map)
    
    PyArg_ParseTuple((input,), "t#:fastcharmap", &s, &n)
    uo = PyUnicode_FromUnicode(NULL, n)
    u = PyUnicode_AsUnicode(uo)
    
    j = 0
    for i from 0 <= i < n:
        uc = m[<unsigned>s[i]]
        if uc == 0xFFFD:
            # The sophisticated unicode_decode_call_errorhandler() is not available to us.
            if errors == 'strict':
                raise ValueError, "fastcharmap can't decode byte 0x%x in position %d" % (s[i], i)
            elif errors == 'ignore':
                continue
            elif errors == 'replace':
                pass
            else:
                raise NotImplementedError, """fastcharmap can't do "%s" error type""" % (str(errors))
        u[j] = uc
        j = j + 1
    
    if j < n:
        uo = PyUnicode_FromUnicode(u, j)
    
    return (uo, n)

def _encode(input, errors, encoding_map):
    cdef char *s
    cdef Py_UNICODE *u, *m, uc
    cdef int i, n, hashi, li, cn, j
    
    try:
        cache = encoding_map._cache
    except:
        if _chatty[0] >= 2:
            print "_orig_encode (not hooked)"
        return _orig_encode(input, errors, encoding_map)
    
    if cache is None:
        cache = _make_encode_cache(encoding_map)
        encoding_map._cache = cache
    
    try:
        m = PyUnicode_AsUnicode(cache)
        if PyUnicode_GetSize(cache) != 3*256:
            raise Exception
    except:
        if _chatty[0] >= 2:
            print "_orig_encode (no cache)"
        return _orig_encode(input, errors, encoding_map)
    
    uo = PyUnicode_FromObject(input)
    u = PyUnicode_AsUnicode(uo)
    n = PyUnicode_GetSize(uo) #len(uo)
    so = PyString_FromStringAndSize(NULL, n)
    s = so
    
    j = 0
    for i from 0 <= i < n:
        uc = u[i]
        hashi = uc & 0xFF
        li = m[hashi]
        cn = -1
        while 1 <= li <= 0x100:
            li = (li-1)*2 + 0x100
            if m[li] == uc:
                cn = m[li+1] & 0xFF
                break;
            li = (m[li+1] >> 8) & 0xFF
        if cn  < 0:
            # The sophisticated charmap_encoding_error() is not available to us.
            if errors == 'strict':
                raise ValueError, "fastcharmap can't encode byte 0x%x in position %d" % (uc, i)
            elif errors == 'ignore':
                continue
            elif errors == 'replace':
                cn = c'?'
            else:
                raise NotImplementedError, """fastcharmap can't do "%s" error type""" % (str(errors))
        s[j] = cn
        j = j + 1
    
    if j < n:
        so = PyString_FromStringAndSize(s, j)
    
    return (so, n)
