from distutils.core import setup
from distutils.extension import Extension
from fastcharmap import __author__, __email__, __url__, __version__
import sys, os

try:
	from Pyrex.Distutils import build_ext
except ImportError:
    ext = {"ext_modules": [Extension("_fastcharmap", ["_fastcharmap.c"])]}
else:
    ext = { "ext_modules": [ Extension( "_fastcharmap",
                                        ["_fastcharmap.pyx"],
                                        extra_compile_args=["-O3"] ) ],
            "cmdclass": {"build_ext": build_ext} }

long_desc="""Fastcharmap is about 5 times faster than Python's standard charmap routines.

Usage:  fastcharmap.hook('codec_name')

Fastcharmap inserts faster code that works when unicode() or "".encode are
called.  Installation requires a binary distribution or a C compiler; Pyrex is
required for changes to the source.  Python's developers are working on faster
charmap codecs for a future version of Python; fastcharmap may be useful until
then.
"""

# patch distutils if it can't cope with the "classifiers" or
# "download_url" keywords
if sys.version < '2.2.3':
    from distutils.dist import DistributionMetadata
    DistributionMetadata.classifiers = None
    DistributionMetadata.download_url = None

setup( name="fastcharmap",
       version=__version__,
       author=__author__,
       author_email=__email__,
       url=__url__,
       description="Faster (5x) charmap codec decode and encode in Pyrex",
       long_description=long_desc,
       license="LGPL",
       platforms=["Python >= 2.3"], # tell me if it works on earlier versions!
       classifiers = [
            "Development Status :: 3 - Alpha",
            "Intended Audience :: Developers",
            "License :: OSI Approved :: GNU Library or Lesser General Public License (LGPL)",
            "Operating System :: OS Independent",
            "Programming Language :: Python",
            "Programming Language :: Other",
            "Programming Language :: C",
            "Topic :: Software Development :: Internationalization",
            ],
       py_modules=["fastcharmap"],
       **ext )
